// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sim.h"
#include <math.h>
double min_thermal_error=2e-9;

void update_heat(struct device *in)
{
/*
int i;
double Ecl=0.0;
double Ecr=0.0;
double Evl=0.0;
double Evr=0.0;
double yl=0.0;
double yr=0.0;
double dh=0.0;
double Jn=0.0;
double Jp=0.0;

for (i=0;i<in->ymeshpoints;i++)
{
	if (i==0)
	{
		Ecl=-in->Xi[0]-in->Vapplied;
		Evl=-in->Xi[0]-in->Vapplied-in->Eg[0];
		yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);
	}else
	{
		Ecl=in->Ec[i-1];
		Evl=in->Ev[i-1];
		yl=in->ymesh[i-1];
	}

	if (i==in->ymeshpoints-1)
	{
		Ecr=-in->Xi[i]-in->Vr;
		Evr=-in->Xi[i]-in->Vr-in->Eg[i];
		yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);

	}else
	{
		Ecr=in->Ec[i];
		Evr=in->Ev[i];
		yr=in->ymesh[i];
	}

	dh=yr-yl;
	Jn=in->Jn[i];
	Jp=in->Jp[i];

	in->He[i]=(Ecr-Ecl)*Jn/dh+in->Hasorb[i]/2.0;
	in->Hh[i]=(Evr-Evl)*Jp/dh+in->Hasorb[i]/2.0;
	//printf("%e\n",in->Hasorb[i]);
	//getchar();
	in->ke[i]=(5/2+1.5)*kb*(kb/q)*in->Te[i]*in->mun[i]*in->n[i];
	in->kh[i]=(5/2+1.5)*kb*(kb/q)*in->Th[i]*in->mup[i]*in->p[i];

	//printf("%e\n",in->He[i]);
	//getchar();
}
*/
}

void dump_thermal(struct device *in)
{
/*int i;
FILE *out;
out=fopen("./thermal.dat","w");
for (i=0;i<in->ymeshpoints;i++)
{
	fprintf(out,"%e %e %e %e %e %e %e %e %e\n",in->ymesh[i],in->Tl[i],in->Te[i],in->Th[i],in->Hl[i],in->He[i],in->Hh[i],in->Hasorb[i],in->kl[i]);
}
fclose(out);
*/
}

double get_thermal_error(struct device *in,double *b)
{
/*
double l=0.0;
double e=0.0;
double h=0.0;

int i;

for (i=0;i<in->ymeshpoints;i++)
{
	l+=mod(b[i]);
	e+=mod(b[i+in->ymeshpoints]);
	h+=mod(b[i+2*in->ymeshpoints]);
}
//printf("%e %e %e\n",l,e,h);
in->tconverge=fopen("./tconverge.dat","a");
fprintf(in->tconverge,"%e\n",l+e+h);
fclose(in->tconverge);
return l+e+h;
*/
}


int solve_thermal(struct device *in)
{
/*
FILE *ttmatrix;
printf("Solve thermal\n");
int i;

int N=in->ymeshpoints*3-2;		//dTldTl
N+=in->ymeshpoints*3-2;			//dTedTe
N+=in->ymeshpoints*3-2;			//dThdTh
N+=in->ymeshpoints;			//dTedTl
N+=in->ymeshpoints;			//dThdTl
N+=in->ymeshpoints;			//dTldTe
N+=in->ymeshpoints;			//dTldTh


int M=in->ymeshpoints*3;
int *Ti =    malloc(N*sizeof(int));
int *Tj =    malloc(N*sizeof(int));
double *Tx = malloc(N*sizeof(double));
double *b = malloc(M*sizeof(double));


double Tll;
double Tlc;
double Tlr;
double Tel;
double Tec;
double Ter;
double Thl;
double Thc;
double Thr;
double yl;
double yc;
double yr;
double dyl=0.0;
double dyc=0.0;
double dyr=0.0;
int ittr=0;
int pos=0;
double error;
double kll=0.0;
double klc=0.0;
double klr=0.0;
double kl0=0.0;
double kl1=0.0;
double dTll=0.0;
double dTlc=0.0;
double dTlr=0.0;
double dTl=0.0;
double Hl=0.0;

double kel=0.0;
double kec=0.0;
double ker=0.0;
double ke0=0.0;
double ke1=0.0;
double dTel=0.0;
double dTec=0.0;
double dTer=0.0;
double dTe=0.0;
double He=0.0;

double khl=0.0;
double khc=0.0;
double khr=0.0;
double kh0=0.0;
double kh1=0.0;
double dThl=0.0;
double dThc=0.0;
double dThr=0.0;
double dTh=0.0;
double Hh=0.0;
double p=0.0;
double dTedTl=0.0;
double dThdTl=0.0;
double dTldTe=0.0;
double dTldTh=0.0;

double Jnl=0.0;
double Jnc=0.0;
double Jnr=0.0;

double Jpl=0.0;
double Jpc=0.0;
double Jpr=0.0;

double mel=0.0;
double mec=0.0;
double mer=0.0;

double mhl=0.0;
double mhc=0.0;
double mhr=0.0;

double Ecl=0.0;
double Ecc=0.0;
double Ecr=0.0;

double Evl=0.0;
double Evc=0.0;
double Evr=0.0;

double xnl=0.0;
double xnc=0.0;
double xnr=0.0;

double xpl=0.0;
double xpc=0.0;
double xpr=0.0;

double munl=0.0;
double munc=0.0;
double munr=0.0;

double mupl=0.0;
double mupc=0.0;
double mupr=0.0;

double phil=0.0;
double phic=0.0;
double phir=0.0;

double tpl=0.0;
double tpc=0.0;
double tpr=0.0;

double tnl=0.0;
double tnc=0.0;
double tnr=0.0;

double nl=0.0;
double nc=0.0;
double nr=0.0;

double dnl=0.0;
double dnc=0.0;
double dnr=0.0;

double dnldTe=0.0;
double dncdTe=0.0;
double dnrdTe=0.0;

double dpldTh=0.0;
double dpcdTh=0.0;
double dprdTh=0.0;

double pl=0.0;
double pc=0.0;
double pr=0.0;

double dpl=0.0;
double dpc=0.0;
double dpr=0.0;

double sxil=0.0;
double sxir=0.0;
double dsxil=0.0;
double dsxir=0.0;


double Snl;
double Snr;
double Snl_pre=0.0;
double Snr_pre=0.0;
double dSnl_predTe=0.0;
double dSnr_predTe=0.0;
double ddh=0.0;
in->thermal_conv=FALSE;
update_heat(in);

	do
	{
if (in->stop==TRUE)
{
ttmatrix=fopen("./ttmatrix.dat","w");
}
		pos=0;
		for (i=0;i<in->ymeshpoints;i++)
		{


			if (i==0)
			{
				mel=in->me[0];
				mhl=in->mh[0];

				kll=in->kl[0];
				kel=in->ke[0];
				khl=in->kh[0];
				//printf("%d\n",in->Tliso);
				//getchar();
				if (in->Tliso==FALSE)
				{
					Tel=in->Tll;
					Thl=in->Tll;
					Tll=in->Tll;
				}else
				{
					Tll=in->Tl[0];
					Tel=in->Tl[0];
					Thl=in->Tl[0];
				}


				yl=in->ymesh[0]-(in->ymesh[1]-in->ymesh[0]);
				Jnl=in->Jn[0];
				Jpl=in->Jp[0];
				Ecl=-in->Xi[0]-phil;
				Evl=-in->Xi[0]-phil-in->Eg[0];
				xnl=in->Fi[0];
				tnl=in->Xi[0];
				nl=in->Nc[i]*exp((q*(xnl+tnl))/(kb*Tel));
				dnl=(q/(kb*Tel))*in->Nc[i]*exp((q*(xnl+tnl))/(kb*Tel));
				dnldTe=0.0;//-((q*(xnl+tnl))/kb)*in->Nc[i]*exp((q*(xnl+tnl))/(kb*Tel))*pow(Tel,-2.0);
				dpldTh=0.0;
				xpl=-in->Fi[0];
				tpl=in->Xi[0]+in->Eg[0];
				pl=in->Nv[i]*exp((q*(xpl-tpl))/(kb*Thl));
				dpl=(q/(kb*Thl))*in->Nv[i]*exp((q*(xpl-tpl))/(kb*Thl));
				munl=in->mun[0];
				mupl=in->mup[0];

			}else
			{
				kll=in->kl[i-1];
				kel=in->ke[i-1];
				khl=in->kh[i-1];
				Tll=in->Tl[i-1];
				Tel=in->Te[i-1];
				Thl=in->Th[i-1];
				yl=in->ymesh[i-1];
				Jnl=in->Jn[i-1];
				Jpl=in->Jp[i-1];

				mel=in->me[i-1];
				mhl=in->mh[i-1];
				phil=in->phi[i-1];
				yl=in->ymesh[i-1];
				Ecl=in->Ec[i-1];
				Evl=in->Ev[i-1];
				nl=in->n[i-1];

				dnl=in->dn[i-1];
				dnldTe=-(q*(in->x[i-1]+in->t[i-1])/kb)*in->Nc[i-1]*exp((q*(in->x[i-1]+in->t[i-1]))/(kb*in->Te[i-1]))*pow(in->Te[i-1],-2.0);
				dpldTh=-(q*(in->xp[i-1]-in->tp[i-1])/kb)*in->Nv[i-1]*exp(((in->xp[i-1]-in->tp[i-1])*q)/(kb*in->Th[i-1]))*pow(in->Th[i-1],-2.0);
				pl=in->p[i-1];
				dpl=in->dp[i-1];
				munl=in->mun[i-1];
				mupl=in->mup[i-1];


			}

			Ecc=-in->Xi[i]-in->phi[i];
			Evc=-in->Xi[i]-in->phi[i]-in->Eg[i];


			if (i==(in->ymeshpoints-1))
			{
				klr=in->kl[in->ymeshpoints-1];
				ker=in->ke[in->ymeshpoints-1];
				khr=in->kh[in->ymeshpoints-1];
				Tlr=in->Tlr;
				Ter=in->Tlr;
				Thr=in->Tlr;
				Jnr=in->Jn[in->ymeshpoints-1];
				Jpr=in->Jp[in->ymeshpoints-1];

				mer=in->me[i];
				mhr=in->mh[i];

				phir=in->Vr;
				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);

				Ecr=-in->Xi[i]-phir;
				Evr=-in->Xi[i]-phir-in->Eg[i];

				xnr=phir+in->Fi[i];
				tnr=in->Xi[i];
				nr=in->Nc[i]*exp((q*(xnr+tnr))/(kb*Ter));

				dnr=(q/(kb*Ter))*in->Nc[i]*exp((q*(xnr+tnr))/(kb*Ter));
				dnrdTe=0.0;//-(q*(xnr+tnr)/kb)*in->Nc[i]*exp((q*(xnr+tnr))/(kb*Ter))*pow(Ter,-2.0);


				xpr=-(phir+in->Fi[i]);
				tpr=in->Xi[i]+in->Eg[i];	//This looks very odd
				pr=in->Nv[i]*exp((q*(xpr-tpr))/(kb*Thr));
				dprdTh=-(q*(xpr-tpr)/kb)*in->Nv[i]*exp((q*(xpr-tpr))/(kb*Thr))*pow(Thr,-2.0);
				dpr=(q/(kb*Thr))*in->Nv[i]*exp((q*(xpr-tpr))/(kb*Thr));
				//dpldTh=-(q*(xpr-tpr)/kb)*in->Nv[i]*exp(((xpr-tpr)*q)/(kb*Thr))*pow(Thr,-2.0);
				munr=in->mun[i];
				mupr=in->mup[i];


				yr=in->ymesh[i]+(in->ymesh[i]-in->ymesh[i-1]);
			}else
			{
				klr=in->kl[i+1];
				ker=in->ke[i+1];
				khr=in->kh[i+1];
				Tlr=in->Tl[i+1];
				Ter=in->Te[i+1];
				Thr=in->Th[i+1];
				Jnr=in->Jn[i+1];
				Jpr=in->Jp[i+1];
				yr=in->ymesh[i+1];

				mer=in->me[i+1];
				mhr=in->mh[i+1];

				phir=in->phi[i+1];

				Ecr=in->Ec[i+1];
				Evr=in->Ev[i+1];

				nr=in->n[i+1];
				dnr=in->dn[i+1];
				dnrdTe=-(q*(in->x[i+1]+in->t[i+1])/kb)*in->Nc[i+1]*exp((q*(in->x[i+1]+in->t[i+1]))/(kb*in->Te[i+1]))*pow(in->Te[i+1],-2.0);
				dprdTh=-(q*(in->xp[i+1]-in->tp[i+1])/kb)*in->Nv[i+1]*exp(((in->xp[i+1]-in->tp[i+1])*q)/(kb*in->Th[i+1]))*pow(in->Th[i+1],-2.0);
				pr=in->p[i+1];
				dpr=in->dp[i+1];
				munr=in->mun[i+1];
				mupr=in->mup[i+1];



			}


			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;

			dyc=(dyl+dyr)/2.0;

			klc=in->kl[i];
			kec=in->ke[i];
			khc=in->kh[i];

			kl0=(kll+klc)/2.0;
			kl1=(klc+klr)/2.0;

			ke0=(kel+kec)/2.0;
			ke1=(kec+ker)/2.0;

			kh0=(khl+khc)/2.0;
			kh1=(khc+khr)/2.0;


			Tlc=in->Tl[i];
			Tec=in->Te[i];
			Thc=in->Th[i];
			Hl=in->Hl[i];
			He=in->He[i];
			Hh=in->Hh[i];

			nc=in->n[i];
			p=in->p[i];

			Jnc=in->Jn[i];
			Jpc=in->Jp[i];



			mec=in->me[i];
			mhc=in->mh[i];

			munc=in->mun[i];
			mupc=in->mup[i];

			munl=(munl+munc)/2.0;
			munr=(munr+munc)/2.0;

			mupl=(mupl+mupc)/2.0;
			mupr=(mupr+mupc)/2.0;


			ke0=(kel+kec)/2.0;
			ke1=(kec+ker)/2.0;

			kh0=(khl+khc)/2.0;
			kh1=(khc+khr)/2.0;

			yc=in->ymesh[i];
			dyl=yc-yl;
			dyr=yr-yc;
			ddh=(dyl+dyr)/2.0;
			phic=in->phi[i];
			Tlc=in->Tl[i];
			Tec=in->Te[i];
			Thc=in->Th[i];

			dncdTe=-((q*(in->x[i]+in->t[i]))/kb)*in->Nc[i]*exp((q*(in->x[i]+in->t[i]))/(kb*in->Te[i]))*pow(in->Te[i],-2.0);
			dpcdTh=-(q*(in->xp[i]-in->tp[i])/kb)*in->Nv[i]*exp(((in->xp[i]-in->tp[i])*q)/(kb*in->Th[i]))*pow(in->Th[i],-2.0);

			dnc=in->dn[i];
			dpc=in->dp[i];



/////////////////Gitter/////////////////////////
			dTll=kl0/dyl/dyc;
			dTlc=-(kl0/dyl/dyc+kl1/dyr/dyc);
			dTlr=kl1/dyr/dyc;

			dTl=dTll*Tll+dTlc*Tlc+dTlr*Tlr+(3.0/2.0)*kb*nc*(Tec-Tlc)/in->tau_e+(3.0/2.0)*kb*p*(Thc-Tlc)/in->tau_h;

			dTlc+=-(3.0/2.0)*kb*nc/in->tau_e;
			dTlc+=-(3.0/2.0)*kb*p/in->tau_h;

			dTldTe=(3.0/2.0)*kb*nc/in->tau_e;
			dTldTh=(3.0/2.0)*kb*p/in->tau_h;

			if ((in->Tliso==TRUE)&&(i==0))
			{
				dTlc+=dTll;
				dTll=0.0;
			}

///////////////////Elektronen////////////////////
//printf("%e %e\n",Ecl,Ecc,Ecr);
//getchar();

if (i==0) Ecl=Ecc;

sxil=q*2.0*(Ecc-Ecl)/(kb*(Tec+Tel))-2.0*(3.0/2.0)*(mec-mel)/(mec+mel)+2.0*(Tec-Tel)/(Tec+Tel);
sxir=q*2.0*(Ecr-Ecc)/(kb*(Ter+Tec))-2.0*(3.0/2.0)*(mer-mec)/(mer+mec)+2.0*(Ter-Tec)/(Ter+Tec);
//if (i==0) printf("bandy wandy=%e %e\n",sxil,sxir);
//getchar();
double dsxildTel=-q*2.0*(Ecc-Ecl)/(kb*pow((Tec+Tel),2.0))-2.0*(Tec-Tel)/pow((Tec+Tel),2.0)-2.0/(Tec+Tel);
double dsxildTec=-q*2.0*(Ecc-Ecl)/(kb*pow((Tec+Tel),2.0))-2.0*(Tec-Tel)/pow((Tec+Tel),2.0)+2.0/(Tec+Tel);

double dsxirdTec=-q*2.0*(Ecr-Ecc)/(kb*pow((Ter+Tec),2.0))-2.0*(Ter-Tec)/(pow((Ter+Tec),2.0))-2.0/(Ter+Tec);
double dsxirdTer=-q*2.0*(Ecr-Ecc)/(kb*pow((Ter+Tec),2.0))-2.0*(Ter-Tec)/(pow((Ter+Tec),2.0))+2.0/(Ter+Tec);
//printf("%e %e %e %e %e %e\n",munl,munc,munr,nl,nc,nr);
//getchar();
Snl_pre=(-(5.0/2.0)*munl*pow((kb*(Tec+Tel)/2.0),2.0)/(q*dyl));
Snr_pre=(-(5.0/2.0)*munr*pow((kb*(Ter+Tec)/2.0),2.0)/(q*dyr));
dSnl_predTe=-(5.0/2.0)*munl*kb*kb*(Tec+Tel)/2.0/(q*dyl);
dSnr_predTe=-(5.0/2.0)*munr*kb*kb*(Ter+Tec)/2.0/(q*dyr);

Snl=Snl_pre*(B(-sxil)*nc-B(sxil)*nl);
Snr=Snr_pre*(B(-sxir)*nr-B(sxir)*nc);
//printf("%d %e %e\n",i,Snl,Snr);
//getchar();
		//	dTel=kl0/dyl/dyc;
		//	dTec=-(kl0/dyl/dyc+kl1/dyr/dyc);
		//	dTer=kl1/dyr/dyc;

//printf("%e %e %e %e %e ind\n",Snl_pre,B(-sxil),nc,B(sxil),nl);

			dTe=(Snl-Snr)/ddh-(3.0/2.0)*kb*nc*(Tec-Tlc)/in->tau_e;
			dTedTl=(3.0/2.0)*kb*nc/in->tau_e;


dTel=Snl_pre*(-dB(-sxil)*dsxildTel*nc-dB(sxil)*dsxildTel*nl)/ddh;
dTec=Snl_pre*(-dB(-sxil)*dsxildTec*nc-dB(sxil)*dsxildTec*nl)/ddh-Snr_pre*(-dB(-sxir)*dsxirdTec*nr-dB(sxir)*dsxirdTec*nc)/ddh;
dTer=-Snr_pre*(-dB(-sxir)*dsxirdTer*nr-dB(sxir)*dsxirdTer*nc)/ddh;

//getchar();
dTel+=dSnl_predTe*(B(-sxil)*nc-B(sxil)*nl)/ddh;
dTec+=dSnl_predTe*(B(-sxil)*nc-B(sxil)*nl)/ddh-dSnr_predTe*(B(-sxir)*nr-B(sxir)*nc)/ddh;
dTer+=-dSnr_predTe*(B(-sxir)*nr-B(sxir)*nc)/ddh;

//dTel=0.0;//dSnl_predTe*(B(-sxil)*nc-B(sxil)*nl)/ddh;
//dTec=0.0;//dSnl_predTe*(B(-sxil)*nc-B(sxil)*nl)/ddh-dSnr_predTe*(B(-sxir)*nr-B(sxir)*nc)/ddh;
//dTer=0.0;//-dSnr_predTe*(B(-sxir)*nr-B(sxir)*nc)/ddh;


//if (in->stop!=TRUE)
		dTec+=-(3.0/2.0)*kb*nc/in->tau_e;

if (in->stop==TRUE)
{
fprintf(ttmatrix,"%e %e %e %e %e %e\n",Snl,Snr,dTel,dTec,dTer,-(3.0/2.0)*kb*nc/in->tau_e);
}


			//double dSnldTe=(5.0/2.0)*(kb/q)*Jnl/(dyl+dyr);
			//double dSnrdTe=-(5.0/2.0)*(kb/q)*Jnr/(dyl+dyr);

			//dTel+=dSnldTe;
			//dTer+=dSnrdTe;

			//if ((in->Tliso==TRUE)&&(i==0))
			//{
			//	dTlc+=dTel;
			//	dTel=0.0;
			//}

////////////////Loch////////////////////////////
			dThl=kh0/dyl/dyc;
			dThc=-(kh0/dyl/dyc+kh1/dyr/dyc);
			dThr=kh1/dyr/dyc;

			dTh=dThl*Thl+dThc*Thc+dThr*Thr-(3.0/2.0)*kb*p*(Thc-Tlc)/in->tau_h;

			dThc+=-(3.0/2.0)*kb*p/in->tau_h;
			dThdTl=(3.0/2.0)*kb*p/in->tau_h;

			double Spl=(5.0/2.0)*(kb/q)*Thl*Jpl;
			double Spr=(5.0/2.0)*(kb/q)*Thr*Jpr;
			double dSpdy=-(Spr-Spl)/(dyl+dyr);
			double dSpldTh=(5.0/2.0)*(kb/q)*Jpl/(dyl+dyr);
			double dSprdTh=-(5.0/2.0)*(kb/q)*Jpr/(dyl+dyr);

			dThl+=dSpldTh;
			dThr+=dSprdTh;

			if ((in->Tliso==TRUE)&&(i==0))
			{
				dTlc+=dThl;
				dThl=0.0;
			}

//////////////Und das Rest//////////////////
			if (i!=0)
			{
				Ti[pos]=i;
				Tj[pos]=i-1;
				Tx[pos]=dTll;
				pos++;

				Ti[pos]=in->ymeshpoints+i;
				Tj[pos]=in->ymeshpoints+i-1;
				Tx[pos]=dTel;
				pos++;

				Ti[pos]=2*in->ymeshpoints+i;
				Tj[pos]=2*in->ymeshpoints+i-1;
				Tx[pos]=dThl;
				pos++;


			}

			//lattice
			Ti[pos]=i;
			Tj[pos]=i;
			Tx[pos]=dTlc;
			pos++;

			Ti[pos]=i;
			Tj[pos]=in->ymeshpoints+i;
			Tx[pos]=dTldTe;
			pos++;

			Ti[pos]=i;
			Tj[pos]=2*in->ymeshpoints+i;
			Tx[pos]=dTldTh;
			pos++;


			//electron
			Ti[pos]=in->ymeshpoints+i;
			Tj[pos]=in->ymeshpoints+i;
			Tx[pos]=dTec;
			pos++;


			Ti[pos]=in->ymeshpoints+i;
			Tj[pos]=i;
			Tx[pos]=dTedTl;
			pos++;

			//Hole
			Ti[pos]=2*in->ymeshpoints+i;
			Tj[pos]=2*in->ymeshpoints+i;
			Tx[pos]=dThc;
			pos++;

			Ti[pos]=2*in->ymeshpoints+i;
			Tj[pos]=i;
			Tx[pos]=dThdTl;
			pos++;



			if (i!=(in->ymeshpoints-1))
			{
				Ti[pos]=i;
				Tj[pos]=i+1;
				Tx[pos]=dTlr;
				pos++;

				Ti[pos]=in->ymeshpoints+i;
				Tj[pos]=in->ymeshpoints+i+1;
				Tx[pos]=dTer;
				pos++;

				Ti[pos]=2*in->ymeshpoints+i;
				Tj[pos]=2*in->ymeshpoints+i+1;
				Tx[pos]=dThr;
				pos++;

			}

			b[i]=-(dTl+Hl);
			b[i+in->ymeshpoints]=-(dTe+in->Hasorb[i]/2.0);
			b[i+2*in->ymeshpoints]=-(dTh+dSpdy+Hh);

		}

		solver(M,N,Ti,Tj, Tx,b);

		if (in->stop==TRUE)
		{
			fclose(ttmatrix);
			exit(0);
		}
		for (i=0;i<in->ymeshpoints;i++)
		{
			double clamp=100.0;
			in->Tl[i]+=b[i];///(1.0+fabs(b[i]/clamp/(300.0*kb/q)));
			in->Te[i]+=b[i+in->ymeshpoints]/(1.0+fabs(b[i+in->ymeshpoints]/100/(300.0*kb/q)));
			in->Th[i]+=b[i+2*in->ymeshpoints]/(1.0+fabs(b[i+2*in->ymeshpoints]/clamp/(300.0*kb/q)));

			//in->Tl[i]+=b[i];
			//in->Te[i]+=b[i+in->ymeshpoints];
			//in->Th[i]+=b[i+2*in->ymeshpoints];
		}


		error=get_thermal_error(in,b);
		printf("Thermal error = %e\n",error);
		if ((ittr<2)&&(error<min_thermal_error)) in->thermal_conv=TRUE;
		ittr++;
		dump_thermal(in);
	}while((ittr<1000)&&(error>min_thermal_error));


//for (i=0;i<in->ymeshpoints;i++)
//{
//	if (in->Tl[i]>2000) in->Tl[i]=2000.0;
//	if (in->Te[i]>2000) in->Te[i]=2000.0;
//	if (in->Th[i]>2000) in->Th[i]=2000.0;
//
//}

dump_thermal(in);

free(Ti);
free(Tj);
free(Tx);
free(b);
return 0;
*/
}

